@@ -124,7 +124,8 @@
 						elmo-flag-folder))
   (elmo-object-save
    (expand-file-name ".minfo" (elmo-folder-msgdb-path folder))
-   (elmo-flag-folder-minfo-internal folder))
+   (elmo-flag-folder-minfo-internal folder)
+   elmo-mime-charset)
   (if (elmo-flag-folder-max-number-internal folder)
       (elmo-object-save
        (expand-file-name "max" (elmo-folder-msgdb-path folder))
@@ -186,22 +187,15 @@
   (when numbers
     (let ((dir (elmo-localdir-folder-directory-internal folder))
 	  (new-msgdb (elmo-make-msgdb))
-	  entity (i 0)
-	  (len (length numbers)))
-      (message "Creating msgdb...")
-      (while numbers
-	(when (setq entity (elmo-localdir-msgdb-create-entity
-			    new-msgdb dir (car numbers)))
-	  (elmo-msgdb-append-entity new-msgdb entity
-				    (list (elmo-flag-folder-flag-internal
-					   folder))))
-	(when (> len elmo-display-progress-threshold)
-	  (setq i (1+ i))
-	  (elmo-display-progress
-	   'elmo-flag-folder-msgdb-create "Creating msgdb..."
-	   (/ (* i 100) len)))
-	(setq numbers (cdr numbers)))
-      (message "Creating msgdb...done")
+	  (flags (list (elmo-flag-folder-flag-internal folder)))
+	  entity)
+      (elmo-with-progress-display (elmo-folder-msgdb-create (length numbers))
+	  "Creating msgdb"
+	(dolist (number numbers)
+	  (when (setq entity (elmo-localdir-msgdb-create-entity
+			      new-msgdb dir number))
+	    (elmo-msgdb-append-entity new-msgdb entity flags))
+	  (elmo-progress-notify 'elmo-folder-msgdb-create)))
       new-msgdb)))
 
 (defun elmo-folder-append-messages-*-flag (dst-folder
