@@ -3,7 +3,7 @@
 ;; Copyright (C) 1985, 1986, 1988, 1990, 1991, 1992, 1993,
 ;;               1994, 1995, 1996, 1997 Free Software Foundation, Inc.
 ;; Copyright (C) 1999 Yoshiki Hayashi <yoshiki@xemacs.org>
-;; Copyright (C) 2000, 2001, 2002 TAKAHASHI Kaoru <kaoru@kaisei.org>
+;; Copyright (C) 2000 TAKAHASHI Kaoru <kaoru@kaisei.org>
 
 ;; Author: TAKAHASHI Kaoru <kaoru@kaisei.org>
 ;;	Yoshiki Hayashi <yoshiki@xemacs.org>
@@ -24,8 +24,8 @@
 
 ;; You should have received a copy of the GNU General Public License
 ;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-;; Boston, MA 02111-1307, USA.
+;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+;; Boston, MA 02110-1301, USA.
 
 ;;; Commentary:
 
@@ -52,10 +52,13 @@
 ;;  @deftypeivar
 ;;  @deftypeop
 ;;  @allowcodebreaks
+;;  @thischapternum
+;;  @quotedblleft @quotedblright
+;;  @quoteleft @quoteright  @quotedblbase @quotesinglbase
+;;  @guillemetleft @guillemetright @guilsinglleft @guilsinglright.
 
 ;;; Code:
 
-(require 'backquote)
 (require 'texinfmt)
 
 ;;; Broken
@@ -69,50 +72,50 @@ This is last argument in `ptexinfmt-broken-facility'.")
   "Declare a symbol FACILITY is broken if ASSERTION is nil.
 DOCSTRING will be printed if ASSERTION is nil and
 `ptexinfmt-disable-broken-notice-flag' is nil."
-  (` (let ((facility '(, facility))
-	   (docstring (, docstring))
-	   (assertion (eval '(, assertion))))
-       (put facility 'broken (not assertion))
-       (if assertion
+  `(let ((facility ',facility)
+	 (docstring ,docstring)
+	 (assertion (eval ',assertion)))
+     (put facility 'broken (not assertion))
+     (if assertion
+	 nil
+       (put facility 'broken-docstring docstring)
+       (if ptexinfmt-disable-broken-notice-flag
 	   nil
-	 (put facility 'broken-docstring docstring)
-	 (if ptexinfmt-disable-broken-notice-flag
-	     nil
-	   (message "BROKEN FACILITY DETECTED: %s" docstring))))))
+	 (message "BROKEN FACILITY DETECTED: %s" docstring)))))
 
 (put 'ptexinfmt-defun-if-broken 'lisp-indent-function 'defun)
 (defmacro ptexinfmt-defun-if-broken (&rest args)
   "Redefine a function just like `defun' if it is considered broken."
   (let ((name (list 'quote (car args))))
     (setq args (cdr args))
-    (` (prog1
-	   (, name)
-	 (if (get (, name) 'broken)
-	     (defalias (, name)
-	       (function (lambda (,@ args)))))))))
+    `(prog1
+	 ,name
+       (if (get ,name 'broken)
+	   (defalias ,name
+	     (function (lambda ,@args)))))))
 
 (put 'ptexinfmt-defun-if-void 'lisp-indent-function 'defun)
 (defmacro ptexinfmt-defun-if-void (&rest args)
   "Define a function just like `defun' unless it is already defined."
   (let ((name (list 'quote (car args))))
     (setq args (cdr args))
-    (` (prog1
-	   (, name)
-	 (if (fboundp (, name))
-	     nil
-	   (defalias (, name)
-	     (function (lambda (,@ args)))))))))
+    `(prog1
+	 ,name
+       (if (fboundp ,name)
+	   nil
+	 (defalias ,name
+	   (function (lambda ,@args)))))))
 
 (put 'ptexinfmt-defvar-if-void 'lisp-indent-function 'defun)
 (defmacro ptexinfmt-defvar-if-void (&rest args)
   "Define a variable just like `defvar' unless it is already defined."
   (let ((name (car args)))
     (setq args (cdr args))
-    (` (prog1
-	   (defvar (, name))
-	 (if (boundp '(, name))
-	     nil
-	   (defvar (, name) (,@ args)))))))
+    `(prog1
+	 (defvar ,name)
+       (if (boundp ',name)
+	   nil
+	 (defvar ,name ,@args)))))
 
 ;; sort -fd
 (ptexinfmt-broken-facility texinfo-format-printindex
@@ -194,6 +197,7 @@ DOCSTRING will be printed if ASSERTION is nil and
 (put 'afourwide 'texinfo-format 'texinfo-discard-line)
 (put 'afivepaper 'texinfo-format 'texinfo-discard-line)
 (put 'pagesizes 'texinfo-format 'texinfo-discard-line-with-args)
+(put 'fonttextsize 'texinfo-format 'texinfo-discard-line-with-args)
 
 ;; style
 (put 'setchapternewpage 'texinfo-format 'texinfo-discard-line-with-args)
@@ -291,7 +295,6 @@ DOCSTRING will be printed if ASSERTION is nil and
 (put 'ifnotdocbook 'texinfo-format 'texinfo-discard-line)
 (put 'ifnotdocbook 'texinfo-end 'texinfo-discard-command)
 
-
 ;; @ifnotinfo ... @end ifnotinfo (makeinfo 3.11 or later)
 (put 'ifnotinfo 'texinfo-format 'texinfo-format-ifnotinfo)
 (ptexinfmt-defun-if-void texinfo-format-ifnotinfo ()
@@ -334,7 +337,6 @@ DOCSTRING will be printed if ASSERTION is nil and
 		 (progn (re-search-forward "@end ifdocbook[ \t]*\n")
 			(point))))
 
-
 
 ;;; Marking
 ;; @indicateurl, @url, @env, @command, 
@@ -520,7 +522,7 @@ For example, @verb\{|@|\} results in @ and
   (goto-char texinfo-command-start))
 
 ;; @,{c}	==>	c,	cedilla accent
-(put ', 'texinfo-format 'texinfo-format-cedilla-accent)
+(put '\, 'texinfo-format 'texinfo-format-cedilla-accent)
 (ptexinfmt-defun-if-void texinfo-format-cedilla-accent ()
   (insert (texinfo-parse-arg-discard) ",")
   (goto-char texinfo-command-start))
@@ -601,6 +603,12 @@ For example, @verb\{|@|\} results in @ and
 (ptexinfmt-defun-if-void texinfo-format-\/ ()
   (texinfo-discard-command))
 
+;; @textdegree
+(put 'textdegree 'texinfo-format 'texinfo-format-textdegree)
+(ptexinfmt-defun-if-void texinfo-format-textdegree ()
+  (insert "o" (texinfo-parse-arg-discard))
+  (goto-char texinfo-command-start))
+
 
 ;;; Cross References
 ;; @ref, @xref
@@ -952,6 +960,40 @@ This command is executed when texinfmt sees @item inside @multitable."
 	(texinfo-sort-region opoint (point))
       (shell-command-on-region opoint (point) "sort -fd" 1))))
 
+
+;; @copying ... @end copying
+;; that Emacs 21.4 and lesser and XEmacs don't support.
+(if (fboundp 'texinfo-copying)
+    nil
+  (defvar texinfo-copying-text ""
+    "Text of the copyright notice and copying permissions.")
+
+  (defun texinfo-copying ()
+    "Copy the copyright notice and copying permissions from the Texinfo file,
+as indicated by the @copying ... @end copying command;
+insert the text with the @insertcopying command."
+    (let ((beg (progn (beginning-of-line) (point)))
+	  (end  (progn (re-search-forward "^@end copying[ \t]*\n") (point))))
+      (setq texinfo-copying-text
+	    (buffer-substring-no-properties
+	     (save-excursion (goto-char beg) (forward-line 1) (point))
+	     (save-excursion (goto-char end) (forward-line -1) (point))))
+      (delete-region beg end)))
+
+  (defun texinfo-insertcopying ()
+    "Insert the copyright notice and copying permissions from the Texinfo file,
+which are indicated by the @copying ... @end copying command."
+    (insert (concat "\n" texinfo-copying-text)))
+
+  (defadvice texinfo-format-scan (before expand-@copying-section activate)
+    "Extract @copying and replace @insertcopying with it."
+    (goto-char (point-min))
+    (when (search-forward "@copying" nil t)
+      (texinfo-copying))
+    (while (search-forward "@insertcopying" nil t)
+      (delete-region (match-beginning 0) (match-end 0))
+      (texinfo-insertcopying))))
+
 (provide 'ptexinfmt)
 
 ;;; ptexinfmt.el ends here
