@@ -220,7 +220,7 @@ Entering Plugged mode calls the value of `wl-plugged-mode-hook'."
        (if (, plugged) wl-plugged-plug-on wl-plugged-plug-off))))
 
 (defmacro wl-plugged-server-indent ()
-  (` (make-string wl-plugged-server-indent ? )))
+  '(make-string wl-plugged-server-indent ? ))
 
 (defun wl-plugged-set-variables ()
   (setq wl-plugged-sending-queue-alist
@@ -720,7 +720,8 @@ Entering Plugged mode calls the value of `wl-plugged-mode-hook'."
        (make-face (intern
 		   (format "wl-highlight-summary-%s-flag-face" (car spec))))
        (nth 1 spec)))
-    (setq elmo-get-folder-function #'wl-folder-make-elmo-folder)
+    (setq elmo-get-folder-function #'wl-folder-make-elmo-folder
+	  elmo-progress-callback-function #'wl-progress-callback-function)
     (setq elmo-no-from wl-summary-no-from-message)
     (setq elmo-no-subject wl-summary-no-subject-message)
     (elmo-global-flags-initialize (mapcar 'car wl-summary-flag-alist))
